import sys

class Progress(object):
    def __init__(self, step=None, total=None, stop=None, name='items'):
        if not total and not step:
            raise Exception('Both step and total arguments are None')
        if total and not step:
            step = int(total / 20)
        self.step = step
        self.count = 0
        self.total = total
        self.stop = stop
        self.name = name
    
    def tick(self):
        self.count += 1
        if not self.count % self.step:
            if self.total:
                percents = ' [%d%%]' % int((self.count / float(self.total)) * 100)
            else:
                percents = ''
            print 'Processed %d %s%s' % (self.count, self.name, percents)
        if self.count == self.stop:
            print 'Reached stop value %d' % self.stop
            sys.exit()



# -*- coding: utf-8
"""
Template tags:
 * alter_qs - allows to modify arguement in URL query string
 * set - allows to set new variable in template

Credits:
 * code for `set` tag is based on http://djangosnippets.org/snippets/215/
"""
from cgi import parse_qsl
from urllib import urlencode

from django import template
from django.utils.encoding import smart_str


register = template.Library()

@register.simple_tag
def alter_qs(qs, name, value, name2=None, value2=None):
    """
    Alter query string argument with new value.

    Usage in template:
        <a href="/foo/bar{% alter_qs request.META.QUERY_STRING "page" 3 %}">Some link</a>

    Usage in code:
        from common.templatetags.common_tags import alter_qs
        url = alter_qs(some_url, "page", "4")
    """

    qs = qs.lstrip('?')
    args = [[x[0], smart_str(x[1])] for x in parse_qsl(qs, keep_blank_values=True)]
    if value:
        found = False
        for arg in args:
            if arg[0] == name:
                arg[1] = smart_str(value)
                found = True
        if not found:
            args.append([name, smart_str(value)])
    else:
        args = [x for x in args if x[0] != name]
    if args:
        args = [tuple(x) for x in args]
        result = '?' + urlencode(args)
        if name2 and value2:
            return alter_qs(result, name2, value2)
        else:
            return result
    else:
        return ''



@register.tag(name='set')
def do_set_varable(parser,token):
    """
    Set new value in the context of template.

    Usage:
        {% set category_list category.categories.all %}
        {% set dir_url "../" %}
        {% set type_list "table" %}
        {% include "category_list.html" %}
    """
    from re import split
    bits = split(r'\s+', token.contents, 2)
    return SetVariableNode(bits[1],bits[2])


class SetVariableNode(template.Node):
    """
    Template Node of `set` tag.
    """

    def __init__(self,varname,value):
        self.varname = varname
        self.value = value
    
    def render(self,context):
        var = template.resolve_variable(self.value,context)
        if var:
            context[self.varname] = var
        else:
            context[self.varname] = context[self.value]
        return ''



import re

from django.conf import settings

from easy_thumbnails.fields import ThumbnailerImageField

class ThumbnailedModel(object):
    def __getattr__(self, name):
        """
        Handles request to FOO_thumbnail and FOO_thumbnail_TYPE fields.
        """
        match = re.search(r'^(\w+)_thumbnail$', name)

        if match:
            field_name = match.group(1)
            size_name = 'normal'
        else:
            match = re.search(r'^(\w+)_thumbnail_(\w+)$', name)
            if match:
                field_name = match.group(1)
                size_name = match.group(2)
            else:
                raise AttributeError('Attribute %s does not exists' % name)

        size_key = ('%s_%s_%s_thumbnail_size' % (
            self._meta.app_label, self._meta.object_name, size_name)).upper()
        size = getattr(settings, size_key)
        try:
            return getattr(self, field_name).get_thumbnail({'size': size, 'crop': 'center'})
        except IOError:
            return None



import string
from random import choice, randint
import itertools

LOREM = "Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi."""
LOREM_ITER = itertools.cycle(LOREM.split(' '))


def random_ip():
    "Generate random ip address."

    x = lambda: randint(1, 255)
    return '%d.%d.%d.%d' % (x(), x(), x(), x())


def random_string(minlen=5, maxlen=8):
    """
    Generate random string of length between ``minlen`` and ``maxlen``.
    """

    length = randint(minlen, maxlen)
    return ''.join(choice(string.lowercase) for x in xrange(length))


def random_text(count):
    """
    Generate random text of ``count`` length.
    """

    text = ' '.join(LOREM_ITER.next() for x in xrange(count))
    return text[0].upper() + text[1:]


def random_html(count):
    """
    Generate random HTML of ``count`` tokens length.
    """

    def tag(word):
        var = randint(0, 20)
        if var == 0:
            return '<b>%s</b>' % word
        elif var == 1:
            return '<i>%s</i>' % word
        elif var == 2:
            return '<a href="http://yandex.ru">%s</a>' % word
        else:
            return word
    text = ' '.join(tag(LOREM_ITER.next()) for x in xrange(count))
    return text[0].upper() + text[1:]


def random_phone():
    def number(size):
        return ''.join(choice(string.digits) for x in xrange(size))
    return '+%s %s %s' % (number(1), number(3), number(7))



"""
This middleware based on http://code.google.com/p/django-profiling/source/browse/trunk/profiling/middleware.py"""

import sys
import tempfile
import hotshot
import hotshot.stats
from cStringIO import StringIO
try:
    from guppy import hpy
    guppy_installed = True
except ImportError:
    guppy_installed = False

from django.conf import settings
from django.http import HttpResponse

class ProfileMiddleware(object):
    """
    Displays hotshot profiling for any view.
    http://yoursite.com/yourview/?prof

    Add the "prof" key to query string by appending ?prof (or &prof=)
    and you'll see the profiling results in your browser.
    It's set up to only be available in django's debug mode,
    but you really shouldn't add this middleware to any production configuration.
    * Only tested on Linux
    """
    def process_request(self, request):
        if 'prof' in request.GET:
            self.tmpfile = tempfile.NamedTemporaryFile()
            self.prof = hotshot.Profile(self.tmpfile.name)

    def process_view(self, request, callback, callback_args, callback_kwargs):
        if 'prof' in request.GET:
            return self.prof.runcall(callback, request, *callback_args, **callback_kwargs)

    def process_response(self, request, response):          
        if 'prof' in request.GET:
            if guppy_installed:
                h = hpy()
                mem_profile = h.heap()
            else:
                mem_profile = 'guppy package is not installed'
            
            self.prof.close()

            out = StringIO()
            old_stdout = sys.stdout
            sys.stdout = out

            stats = hotshot.stats.load(self.tmpfile.name)
            #stats.strip_dirs()
            stats.sort_stats('cumulative')
            stats.print_stats()

            sys.stdout = old_stdout
            stats_str = out.getvalue()

            if response and response.content and stats_str:
                out = response.content.split('</body>')
                tail = """
                    <div style="text-align: left">
                        <h1>Instance wide RAM usage</h1>
                        <pre>%s</pre>
                        <br/><br/><br/>
                        <h1>CPU Time for this request</h1>
                        <pre>%s</pre>
                    </div>
                """ % (mem_profile, stats_str)
                out = out[0] + tail + out[1]
                return HttpResponse(out)
                    
        return response



# -*- coding: utf-8 -*-
from django.shortcuts import redirect, get_object_or_404
from django.core.urlresolvers import reverse

from common.decorators import render_to
from common.pagination import paginate



import os
import sys

def setup_django(script_file, relpath=None):
    ROOT = os.path.dirname(os.path.realpath(script_file))
    if relpath:
        ROOT = os.path.join(ROOT, relpath)
    os.chdir(ROOT)
    sys.path.append(ROOT)
    os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'



# -*- coding: utf-8 -*-
"""
Contains view's decorators which automates common tasks.
"""

import traceback

from django.shortcuts import render_to_response
from django.template import RequestContext

from common.http import HttpResponseJson

def render_to(template):
    """
    Render view's output with ``template`` using ``RequestContext``.

    If decorated view returns dict object then wrap it in RequestContext and
    render the template.

    If decorated view returns non dict object then just return this object.

    Args:
        :template: path to template
    
    Example::

        @render_to('blog/index.html')
        def post_list(request):
            posts = Post.objects.all()
            return {'posts': posts,
                    }
    """

    def decorator(func):
        def wrapper(request, *args, **kwargs):
            output = func(request, *args, **kwargs)
            if not isinstance(output, dict):
                return output
            else:
                ctx = RequestContext(request)
                return render_to_response(template, output, context_instance=ctx)
        return wrapper
    return decorator


def ajax(func):
    """
    Convert views's output into JSON.

    Decorated view should return dict object.

    If ``request.method`` is not ``POST`` then deny the request.

    If view raises Exception then return JSON message with error description.
    """

    def wrapper(request, *args, **kwargs):
        if request.method == 'POST':
            try:
                response = func(request, *args, **kwargs)
            except Exception, ex:
                response = {'error': traceback.format_exc()}
        else:
            response = {'error': {'type': 403, 'message': 'Accepts only POST request'}}
        if isinstance(response, dict):
            return HttpResponseJson(response)
        else:
            return response
    return wrapper


def ajax_get(func):
    """
    Convert views's output into JSON.

    Decorated view should return dict object.

    If view raises Exception then return JSON message with error description.
    """

    def wrapper(request, *args, **kwargs):
        try:
            response = func(request, *args, **kwargs)
        except Exception, ex:
            response = {'error': traceback.format_exc()}
        if isinstance(response, dict):
            return HttpResponseJson(response)
        else:
            return response
    return wrapper



# -*- coding: utf-8 -*-
"""
This module provides ``Enum`` class which simplifies work with enumerated list of choices.
This implementation is specially developed for use in django models.

Example of usage::

    from common import enum

    class Color(enum.Enum):
        red = enum.Item(1)
        green = enum.Item(2, 'So greeeen')


We defined here two items. They are accessable as ``Color.red`` and ``Color.green``.
``Color.red`` will give you ``1``. So ``Color.red == 1`` is ``True``.

First item (``Color.red``) has the label same to its key i.e., "red". Second item (``Color.green``) has the custom label.  Labels are used when Color object are queried for the (key, label) pairs. This happens, for example, when we use Color object as the value for the ``choices`` argument of django field::

    class Fruit(models.Model):
        color = models.IntegerField(choices=Color, default=Color.red)

We can use Color object as ``choices`` argument because ``enum.Enum`` class provides custom __iter__
method which returs (key, label) pairs.

Also keep in mind that ``Color.red`` is not simple integer. It is like integer but has
some extra methods. Look example:

    Color.green == 2
    Color.green.label == "So greeeen"
    Color.green.key == "green"

Other useful methods of enum.Enum class::

   Color.by_value(1) == Color.red
   Color.by_key("red") == Color.red
   Color.values == [Color.red, Color.green]
   Color.random_value() == "Random value choosed from Color items"

Some tests:

# Common usage of enum module
>>> class Body(Enum):
...     sedan = Item(1, u'Sedan')
...     hatchback = Item(2, u'Hatchback')
>>> set(Body)
set([(1, u'Sedan'), (2, u'Hatchback')])
>>> Body.sedan
1

# Build Enum class from list of tuples
>>> Body = build(((1, u'Sedan'), (2, u'Hatchback')))
>>> set(Body)
set([(1, u'Sedan'), (2, u'Hatchback')])

# Specify items with ``_choices`` attribute
>>> class Body(Enum):
...     _choices = ((1, u'Sedan'), (2, u'Hatchback'))
>>> set(Body)
set([(1, u'Sedan'), (2, u'Hatchback')])

# ``_choices`` also could be a dict instance
>>> class Body(Enum):
...     _choices = dict(Sedan=1, Hatchback=2)
>>> set(Body)
set([(1, 'Sedan'), (2, 'Hatchback')])

# Get enum Item by its value
>>> Body.by_value(1).key
'Sedan'

# Pass arbitrary data to items
>>> class Color(Enum):
...     red = Item(1, 'Red', example='Apple')
...     green = Item(2, 'Green', example='Tree')
>>> Color.green.example
'Tree'
"""

import re
from random import choice

class NotFound(Exception):
    "Raise when could not found item with specified parameters"

def setup_item(obj, value, label, **kwargs):
    obj.value = value
    if label is None:
        obj.label = str(obj)
    else:
        obj.label = label
    for ikey, ivalue in kwargs.iteritems():
        setattr(obj, ikey, ivalue)
    return obj


class IntItem(int):
    def __new__(cls, value, label=None, **kwargs):
        obj =  int.__new__(cls, value)
        return setup_item(obj, value, label, **kwargs)


class StrItem(str):
    def __new__(cls, value, label=None, **kwargs):
        obj =  str.__new__(cls, value)
        return setup_item(obj, value, label, **kwargs)


def Item(value, *args, **kwargs):
    if isinstance(value, int):
        return IntItem(value, *args, **kwargs)
    else:
        return StrItem(value, *args, **kwargs)


def items_from_choices(choices):
    """
    Create dict of enum.Item objects from given values.
    Args:
        choices: dict (key->value) or list of pairs [(value, key), ...]
    """

    items = {}
    if isinstance(choices, dict):
        choices = [(y, x) for x, y in choices.items()]
    for value, label in choices:
        key = label.replace(' ', '_').replace('-', '_')
        key = re.sub(r'_+', '_', key)
        rex = re.compile(r'^[a-z0-9_]*$', re.I)
        if not rex.match(key):
            raise Exception('Could not create key from label: %s' % label)
        items[key] = Item(value, label)
    return items


class MetaEnum(type):
    """
    Find all enum.Item attributes and save them into ``_items`` attribute.
    """

    def __new__(cls, name, bases, attrs):
        items = {}
        for base in bases:
            if isinstance(base, MetaEnum):
                items.update(base._items)
        if '_choices' in attrs:
            attrs.update(items_from_choices(attrs['_choices']))
            del attrs['_choices']
        for key, attr in attrs.items():
            if isinstance(attr, (IntItem, StrItem)):
                attr.key = key
                items[key] = attr
                del attrs[key]
        attrs['_items'] = items
        return type.__new__(cls, name, bases, attrs)

    """
    Public methods:
    """

    def __iter__(cls):
        """
        Iterate over tuples of (value, label)
        """

        return iter(cls.choices())

    def __len__(self):
        """
        Return the number of enum.Item objects.
        """

        return len(self._items)

    def choices(cls):
        """
        Return tuples of (value, label) for all enum.Item objects.
        """

        return [(x.value, x.label) for x in cls._items.itervalues()]

    def values(self):
        """
        Return list of values of all enum.Item objects.
        """

        return self._items.values()

    def random_value(cls):
        """
        Return random value of enum.Item object.
        """

        return choice(cls._items.values())

    """
    Private methods:
    """

    def __getattribute__(self, key):
        """
        Each enum.Item object could be accessed as enum.Enum instance's attribute.
        """

        items = type.__getattribute__(self, '_items')
        if key in items:
            return items[key]
        else:
            if key.startswith('by_'):
                return type.__getattribute__(self, '_by_attribute')(key[3:])
            else:
                return type.__getattribute__(self, key)

    def _by_attribute(cls, attr):
        def func(value):
            """
            Return enum.Item which has attribute with specified value.
            """
            for x in cls._items.itervalues():
                if getattr(x, attr) == value:
                    return x
            raise NotFound('Could not found item which %s attribute is %s' % (attr, value))
        return func

    def by_key(cls, key):
        """
        Return enum.Item which has the given key.
        """

        return cls._items[key]



class Enum(object):
    NotFound = NotFound
    __metaclass__ = MetaEnum


def build(choices):
    class _Enum(Enum):
        _choices = choices
    return _Enum


if __name__ == '__main__':
    import doctest
    doctest.testmod()



def iterate(qs, step=5000):
    """
    Reduce memory usage by splitting query into a number of queries using offset, limit
    """

    total = qs.count()
    for start in xrange(0, total, step):
        for obj in qs[start:start + step]:
            yield obj


def load_related(objects, rel_qs, rel_field_name, cache_field_name=None):
    """
    Load in one SQL query the objects from query set (rel_qs)
    which is linked to objects from (objects) via the field (rel_field_name).
    """

    obj_map = dict((x.id, x) for x in objects)
    rel_field = rel_qs.model._meta.get_field(rel_field_name)
    cache_field_name = '%s_cache' % rel_qs.model.__name__.lower()

    rel_objects = rel_qs.filter(**{'%s__in' % rel_field.name: obj_map.keys()})

    temp_map = {}
    for rel_obj in rel_objects:
        pk = getattr(rel_obj, rel_field.attname )
        temp_map.setdefault(pk, []).append(rel_obj)

    for pk, rel_list in temp_map.iteritems():
        setattr(obj_map[pk], cache_field_name, rel_list)



"""
Django-common pagination module dramatically simplifies the pagination task.

You only have to do following things.

In view::

    @render_to('foo_list.html')
    def foo_list(request):
        qs = Foo.objects.all()
        page = paginate(qs, request)
        return {'page': page,
                }


In template::

    <ul>
    {% for item in page.object_list %}
        <li>{{ item }}</li>
    {% endfor %}
    </ul>

    {% include "pagination.html" %}

If you want to know how this work under the hood please look at source code.
"""

from django.core.paginator import Paginator, EmptyPage, InvalidPage

from common.templatetags.common_tags import alter_qs


def paginate(qs, request, per_page=15, frame_size=10):
    """
    Return extended ``django.core.paginator.Page`` object

    Args:

        :qs: queryset which should be paginated
        :request: django request object
        :per_page: number of objects per page
        :frame_size: number of visible pages (does not include first and large page)
    """

    try:    
        page_number = int(request.GET.get('page', 1))
    except ValueError:
        page_number = 1

    paginator = Paginator(qs, per_page)
    try:    
        page = paginator.page(page_number)
    except (EmptyPage, InvalidPage):
        page_number = 1
        page = paginator.page(1)
    query_string = request.META['QUERY_STRING']

    if page.has_previous():
        page.previous_page_url = alter_qs(query_string, 'page', page.previous_page_number())
    else:
        page.previous_page_url = None

    if page.has_next():
        page.next_page_url = alter_qs(query_string, 'page', page.next_page_number())
    else:
        page.next_page_url = None

    page.first_page_url = alter_qs(query_string, 'page', 1)
    page.last_page_url = alter_qs(query_string, 'page', page.paginator.num_pages)

    urls = []
    if frame_size is None:
        for x in page.paginator.page_range:
            urls.append((x, alter_qs(query_string, 'page', x)))
        start = 1
        end = page.paginator.page_range
    else:
        half = int(frame_size / 2.0)
        start = max(1, page.number - int(frame_size / 2.0))
        stop = min(page.paginator.num_pages, start + frame_size - 1)
        if stop == page.paginator.num_pages:
            if stop - start < (frame_size - 1):
                start = max(1, stop - frame_size)
        if start == 1:
            if stop - start < (frame_size - 1):
                stop = min(page.paginator.num_pages, start + frame_size)
        for x in xrange(start, stop + 1):
            urls.append((x, alter_qs(query_string, 'page', x)))
    page.paginator.frame = urls
    page.paginator.frame_start_page = start
    page.paginator.frame_end_page = stop

    return page



import time
from random import randint
from hashlib import sha1
from tempfile import NamedTemporaryFile
import os
import Image
import ImageDraw

from django.core.files import File

def random_color():
    return 'rgb(%d, %d, %d)' % (randint(0, 255), randint(0, 255), randint(0, 255))


def generate_image(size):
    img = Image.new('RGBA', size=size)
    draw = ImageDraw.Draw(img)
    for x in xrange(30):
        diameter = int(min(*size) / float(randint(4, 8)))
        topleft = (0 + randint(0, size[0]),
                   0 + randint(0, size[1]))
        bottomright = (topleft[0] + diameter,
                       topleft[1] + diameter)
        draw.pieslice((topleft + bottomright), 0, 360, fill=random_color())
    return img


def random_image(size=(200, 200)):
    source = '%d%d' % (time.time(), id({}))
    hashname = sha1(source).hexdigest() + '.jpg'
    tmpfile = NamedTemporaryFile()
    img = generate_image(size)
    img.save(tmpfile, 'JPEG')
    tmpfile.seek(0)
    return File(tmpfile, name=hashname)



# -*- coding: utf-8 -*-
from django import forms

def build_form(Form, _request, *args, **kwargs):
    """
    Build the ``Form`` instance.

    If ``request.method`` is ``POST`` then build bound form else build unbound.

    Usage example::

        @login_required
        @render_to('blog/post_create.html')
        def post_create(request):
            post = Post(instance=request.user)
            form = build_form(PostForm, request, instance=post)
            if form.is_valid():
                obj = form.save()
                return redirect(obj)
            return {'form': form,
                    }
    """

    if 'POST' == _request.method:
        form = Form(_request.POST, _request.FILES, *args, **kwargs)
    else:
        form = Form(*args, **kwargs)
    return form


class DeleteForm(forms.Form):
    """
    This form could be used in conjunction with ``build_form`` on pages where user have to confirm some action.

    Typical example of such page is deletion the some object.

    Fields:
        :next: this is optional field. You can use it to store redirection URL.

    Usage example::

        @render_to('blog/post_delete.html')
        def post_delete(request, postid):
            post = get_object_or_404(Post, pk=postid)
            form = build_form(DeleteForm, request, initial={'next': request.GET.get('next')})
            if form.is_valid():
                post.delete()
                return redirect(form.cleaned_data['next'] or '/')
            return {'form': form,
                    'post': post,
                   }
    """

    next = forms.CharField(required=False, widget=forms.HiddenInput)
